@@ -4,7 +4,7 @@
 
 namespace code {
 
-	Content::Content() {}
+	Content::Content() : cOwner(null) {}
 
 	const void *Content::address() const {
 		return lastAddress;
@@ -14,22 +14,31 @@ namespace code {
 		return lastSize;
 	}
 
-	void Content::set(const void *addr, nat size) {
+	void Content::set(const void *addr, Nat size) {
 		lastAddress = addr;
 		lastSize = size;
 
-		if (owner)
-			owner->update();
+		RefSource *o = owner();
+		if (o)
+			o->update();
 	}
 
 	Str *Content::ownerName() const {
-		RefSource *o = (RefSource *)atomicRead((void *&)owner);
+		RefSource *o = owner();
 		if (o)
 			return o->title();
 		else
 			return null;
 	}
 
+	RefSource *Content::owner() const {
+		return atomicRead(cOwner);
+	}
+
+	RefSource *Content::stolenBy() const {
+		return null;
+	}
+
 	StaticContent::StaticContent(const void *addr) {
 		set(addr, 0);
 	}
