@@ -67,6 +67,10 @@ namespace storm {
 		// To string.
 		virtual void STORM_FN toS(StrBuf *to) const;
 
+		// Called by other parts of the system to invalidate the ordering here. Most commonly done
+		// when objects have been replaced, so that even pinned objects are updated.
+		void invalidate();
+
 	private:
 		// Data.
 		struct Data {
