@@ -54,7 +54,7 @@ namespace os {
 #elif defined(LINUX_IO_URING)
 
 	IORequest::IORequest(Handle handle, const Thread &thread)
-		: thread(thread), result(0), timeout(false) {
+		: thread(thread), result(0), timeout(false), deallocate(false) {
 
 		zeroMem(request);
 		request.fd = handle.v();
@@ -82,6 +82,16 @@ namespace os {
 		return result;
 	}
 
+	void IORequest::submitDetachDelete(nat timeout) {
+		deallocate = true;
+		io().attachAndRemove(Handle(request.fd), this);
+		if (timeout) {
+			sleep.until = UThreadState::sleepTarget(timeout);
+			sleep.request = this;
+			thread.threadData()->uState.addSleep(&sleep);
+		}
+	}
+
 	IORequest::~IORequest() {
 		if (sleep.request)
 			thread.threadData()->uState.cancelSleep(&sleep);
@@ -92,6 +102,18 @@ namespace os {
 		return thread.threadData()->ioComplete;
 	}
 
+	void IORequest::onFinish(int result) {
+		this->result = result;
+		bool deallocate = this->deallocate;
+
+		wake.set();
+
+		// Note: We don't touch 'this' after calling 'set'. If we are stack allocated by another
+		// thread, we might be deallocated by now.
+		if (deallocate)
+			delete this;
+	}
+
 	void IOTimeoutSleep::signal() {
 		if (request) {
 			// Ask to cancel.
